import traceback

from PyQt5 import uic
from PyQt5.QtCore import pyqtSignal, QProcessEnvironment, QProcess
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QApplication, QMessageBox, QMainWindow

from astra_installer.commands import Chroot
from astra_installer.utils import DesktopUtils, User
from astra_installer.utils import Utils as BackendUtils
from astra_installer_qt_oem_integration.utils import Utils

translate = QApplication.translate

mainwindow_ui = Utils.file_path('ui', 'mainwindow.ui')
UiFormClass, UiFormBase = uic.loadUiType(mainwindow_ui)  # type: QMainWindow, QMainWindow

from astra_installer.commands import is_security_feature_enabled


class MainWindow(UiFormClass, UiFormBase):
    oem_enabled_changed = pyqtSignal()

    def __init__(self, hide_eula_page: bool = False) -> None:
        super(MainWindow, self).__init__()
        self.setupUi(self)
        fallbackicon = QIcon.fromTheme('preferences-system')
        icon = QIcon.fromTheme('preferences-desktop-display', fallbackicon)
        self.hide_eula_page = hide_eula_page
        self.setWindowIcon(icon)
        self.pushButtonQuit.clicked.connect(self.quit)
        self.pushButtonEnable.clicked.connect(self.enable_or_disable)
        self.oem_enabled_changed.connect(self.rename_button)
        self.oem_enabled = self.is_oem_enabled()

    @staticmethod
    def is_oem_enabled() -> bool:
        return BackendUtils.is_kiosk_session()

    @property
    def oem_enabled(self) -> bool:
        return self.__oem_enabled

    @oem_enabled.setter
    def oem_enabled(self, value: bool) -> None:
        self.__oem_enabled = value
        self.oem_enabled_changed.emit()

    def rename_button(self) -> None:
        text = ''
        if self.oem_enabled:
            text = translate('text', 'Disable OEM integration mode')
        else:
            text = translate('text', 'Enable OEM integration mode')
        self.pushButtonEnable.setText(text)

    def enable_or_disable(self) -> None:
        if self.ask_confirmation():
            if self.oem_enabled:
                self.disable_oem_integration()
            else:
                if self.check_oem_and_sudo_password():
                    self.enable_oem_integration()

    def check_oem_and_sudo_password(self) -> bool:
        if is_security_feature_enabled('/usr/sbin/astra-sudo-control'):
            title = translate('MainWindowCommon', 'Sudo password prompt warning')
            question = translate('MainWindowCommon',
                                 'Warning: Password prompt is enabled for sudo. '
                                 'In OEM integration mode, the user will not be able to run the initial '
                                 'system configuration utility without a password. Do you want to continue?')
            return QMessageBox.question(self, title, question) == QMessageBox.Yes
        else:
            return True

    def ask_confirmation(self) -> bool:
        title = translate('text', 'Setup confirmation')
        action = ''
        if self.oem_enabled:
            action = translate('text', 'disable OEM integration mode')
        else:
            action = translate('text', 'enable OEM integration mode')
        question = translate('text', 'Are you really sure you want to {action}?')
        return QMessageBox.question(self, title, question.format(action=action)) == QMessageBox.Yes

    def ask_about_reboot(self) -> None:
        title = translate('text', 'The operation completed successfully')
        operation = ''
        if self.oem_enabled:
            operation = translate('text', 'OEM integration mode successfully enabled')
        else:
            operation = translate('text', 'OEM integration mode successfully disabled')
        question = translate('text', '{operation}.\n'
                                     'Changes will take effect after a reboot.\n'
                                     'Do you want to reboot now?')
        answer = QMessageBox.question(self, title, question.format(operation=operation))
        if answer == QMessageBox.Yes:
            self.reboot()

    def reboot(self) -> None:
        env = QProcessEnvironment.systemEnvironment()
        env.insert('XDG_RUNTIME_DIR', '/run/user/{uid}'.format(uid=BackendUtils.get_sudo_uid()))
        process = QProcess()
        process.setProcessEnvironment(env)
        process.setProgram('/usr/bin/fly-wmfunc')
        process.setArguments(['FLYWM_REBOOT'])
        process.startDetached()

    def show_error(self, error: str, details: str = None) -> None:
        title = translate('text', 'Critical error')
        msgbox = QMessageBox(self)
        msgbox.setWindowTitle(title)
        msgbox.setText(error)
        if details:
            msgbox.setDetailedText(details)
        msgbox.exec()

    def enable_oem_integration(self) -> None:
        try:
            utils = DesktopUtils(Chroot('/'))
            utils.install_kiosk_profile('astra-admin', self.hide_eula_page)
            utils.enable_autologin(User(BackendUtils.get_current_user()))
            utils.disable_mac_prompt()
            utils.disable_kiosk_screenlocker()
            self.oem_enabled = True
            self.ask_about_reboot()
        except:
            error = translate('text', 'Failed to enable OEM integration mode')
            self.show_error(error, traceback.format_exc())

    def disable_oem_integration(self) -> None:
        try:
            utils = DesktopUtils(Chroot('/'))
            utils.uninstall_kiosk_profile('astra-admin')
            utils.disable_autologin(User(BackendUtils.get_current_user()))
            utils.enable_mac_prompt()
            utils.enable_kiosk_screenlocker()
            self.oem_enabled = False
            self.ask_about_reboot()
        except:
            error = translate('text', 'Failed to disable OEM integration mode')
            self.show_error(error, traceback.format_exc())

    def quit(self) -> None:
        QApplication.instance().quit()
